"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const matter = require("gray-matter");
const path = require("path");
const recursive = require("recursive-readdir");
const defaultMetaData = {
    lang: "en",
};
const loadJsonFile = (filePath) => JSON.parse(fs.readFileSync(filePath, { encoding: "utf8" }));
const getDefaultSetting = (dirname, opts, filename = "rocu.json") => {
    let globalSetting;
    const filePath = path.join(dirname, filename);
    if (opts.watcher && opts.watcher.filename === filePath) {
        globalSetting = undefined;
    }
    if (globalSetting) {
        return globalSetting;
    }
    if (fs.existsSync(filePath) && fs.statSync(filePath).isFile()) {
        return (globalSetting = loadJsonFile(filePath));
    }
    return defaultMetaData;
};
const rewriteMetaData = (globalSetting, localSetting) => {
    const globalLinks = [...(globalSetting.link ? globalSetting.link : []), ...(globalSetting.css ? globalSetting.css : [])];
    const localLinks = [...(localSetting.link ? localSetting.link : []), ...(localSetting.css ? localSetting.css : [])];
    return {
        ...globalSetting,
        ...localSetting,
        globalScripts: globalSetting.js,
        localScripts: localSetting.js,
        globalLinks,
        localLinks,
    };
};
const getPage = (dirname, opts) => async (filename) => {
    const globalSetting = getDefaultSetting(dirname, opts);
    const ext = path.extname(filename);
    const relativePath = path.relative(dirname, filename);
    const name = relativePath.slice(0, relativePath.length - ext.length);
    const raw = fs.readFileSync(filename, "utf8");
    const { data, content } = matter(raw);
    const metaData = rewriteMetaData(globalSetting, data);
    return {
        content,
        metaData,
        ext,
        filename,
        name,
        raw,
    };
};
const getLayout = (pages) => (page) => {
    if (page.ext !== ".md ") {
        return page;
    }
    if (!page.metaData.layout) {
        return page;
    }
    const layout = pages.find((p) => p.name === page.metaData.layout);
    if (!layout) {
        return page;
    }
    page.metaData = { ...layout.metaData, ...page.metaData };
    page.layoutJSX = layout.content;
    return page;
};
exports.getData = async (dirname, opts) => {
    const allFiles = await recursive(dirname);
    const filenames = allFiles.filter(name => !/^\./.test(name));
    const jsxFilenames = filenames.filter(name => /\.jsx$/.test(name));
    const mdFilenames = filenames.filter(name => /\.mdx?/.test(name));
    const contentFiles = [...jsxFilenames, ...mdFilenames];
    const promises = contentFiles.map(getPage(dirname, opts));
    const pages = await Promise.all(promises);
    const withLayouts = pages.map(getLayout(pages));
    return {
        dirname,
        pages: withLayouts,
    };
};
